@@ -1,10 +1,8 @@
 package Test::Smoke;
 use strict;
 
-# $Id: Smoke.pm 1155 2008-01-03 13:32:28Z abeltje $
-use vars qw( $VERSION $REVISION $conf @EXPORT );
-$VERSION  = '1.29_60';
-$REVISION = __get_ts_patchlevel();
+use vars qw($VERSION $conf @EXPORT);
+$VERSION  = '1.61_03';
 
 use base 'Exporter';
 @EXPORT  = qw( $conf &read_config &run_smoke );
@@ -35,11 +33,13 @@ Test::Smoke - The Perl core test smoke suite
 
 =head1 DESCRIPTION
 
+If you are looking to get started, start at the B<README>!
+
 C<Test::Smoke> exports C<$conf> and C<read_config()> by default.
 
-=over 4
+=head2 Test::Smoke::read_config( $config_name )
 
-=item Test::Smoke::read_config( $config_name )
+Read (require) the configfile.
 
 =cut
 
@@ -59,7 +59,7 @@ sub read_config {
     return defined $ConfigError ? undef : 1;
 }
 
-=item Test::Smoke->config_error()
+=head2 Test::Smoke->config_error()
 
 Return the value of C<$ConfigError>
 
@@ -69,7 +69,7 @@ sub config_error {
     return $ConfigError;
 }
 
-=item is_win32( )
+=head2 is_win32( )
 
 C<is_win32()> returns true if  C<< $^O eq "MSWin32" >>.
 
@@ -77,7 +77,7 @@ C<is_win32()> returns true if  C<< $^O eq "MSWin32" >>.
 
 sub is_win32() { $^O eq "MSWin32" }
 
-=item do_manifest_check( $ddir, $smoker )
+=head2 do_manifest_check( $ddir, $smoker )
 
 C<do_manifest_check()> uses B<Test::Smoke::SourceTree> to do the 
 MANIFEST check.
@@ -98,7 +98,7 @@ sub do_manifest_check {
     }
 }
 
-=item set_smoke_patchlevel( $ddir, $patch[, $verbose] )
+=head2 set_smoke_patchlevel( $ddir, $patch[, $verbose] )
 
 Set the current patchlevel as a registered patch like "SMOKE$patch"
 
@@ -108,11 +108,13 @@ sub set_smoke_patchlevel {
     my( $ddir, $patch, $verbose ) = @_;
     $ddir && $patch or return;
 
-    my @smokereg = grep /^SMOKE\d+$/ => get_local_patches( $ddir, $verbose );
+    my @smokereg = grep
+        /^SMOKE[a-fA-F0-9]+$/
+    , get_local_patches( $ddir, $verbose );
     @smokereg or set_local_patch( $ddir, "SMOKE$patch" );
 }
 
-=item run_smoke( [$continue[, @df_buildopts]] )
+=head2 run_smoke( [$continue[, @df_buildopts]] )
 
 C<run_smoke()> sets up de build environment and gets the private Policy
 file and build configurations and then runs the smoke stuff for all 
@@ -135,6 +137,15 @@ sub run_smoke {
 
     my $patch = Test::Smoke::Util::get_patch( $conf->{ddir} );
 
+    { # I cannot find a better place to stick this (thanks Bram!)
+      # change 33961 introduced Test::Harness 3 for 5.10.x
+      # that needs different parsing, so set the config to do that
+      # 20081220; new patchlevels due to git; cannot test it like an int
+        if ( $conf->{perl_version} eq '5.10.x' ) {
+            $conf->{hasharness3} = 1;
+        }
+    }
+
     my $logfile = File::Spec->catfile( $conf->{ddir}, 'mktest.out' );
     my $BuildCFG = $continue 
         ? Test::Smoke::BuildCFG->continue( $logfile, $conf->{cfg}, 
@@ -165,9 +176,10 @@ sub run_smoke {
     chdir $conf->{ddir} or die "Cannot chdir($conf->{ddir}): $!";
     unless ( $continue ) {
         $smoker->make_distclean( );
-        $smoker->ttylog( "Smoking patch $patch\n" ); 
+        $smoker->ttylog("Smoking patch $patch->[0] $patch->[1]\n"); 
+        $smoker->ttylog("Smoking branch $patch->[2]\n") if $patch->[2];
         do_manifest_check( $conf->{ddir}, $smoker );
-        set_smoke_patchlevel( $conf->{ddir}, $patch );
+        set_smoke_patchlevel( $conf->{ddir}, $patch->[0] );
     }
 
     foreach my $this_cfg ( $BuildCFG->configurations ) {
@@ -182,7 +194,7 @@ sub run_smoke {
         $smoker->smoke( $this_cfg, $Policy );
     }
 
-    $smoker->ttylog( "Finished smoking $patch\n" );
+    $smoker->ttylog( "Finished smoking $patch->[0] $patch->[1] $patch->[2]\n" );
     $smoker->mark_out;
 
     close LOG or do {
@@ -191,33 +203,8 @@ sub run_smoke {
    };
 }
 
-=item __get_ts_patchlevel( )
-
-Read the contents of F<.patch>.
-
-=cut
-
-use FindBin;
-use File::Spec::Functions;
-
-sub __get_ts_patchlevel {
-    my( $rev ) = q$Rev: 1155 $ =~ /(\d+)/;
-    my $dotpatch = catfile $FindBin::Bin, '.patch';
-    local *DOTPATCH;
-    open DOTPATCH, "< $dotpatch" or return $rev;
-    chomp( my $plevel = <DOTPATCH> );
-    close DOTPATCH;
-    return $plevel > $rev ? $plevel : $rev;
-}
-
 1;
 
-=back
-
-=head1 REVISION
-
-$Id: Smoke.pm 1155 2008-01-03 13:32:28Z abeltje $
-
 =head1 COPYRIGHT
 
 (c) 2003, All rights reserved.
