@@ -1,15 +1,14 @@
 package Test::Smoke::Smoker;
+use warnings;
 use strict;
 
-# $Id: Smoker.pm 1155 2008-01-03 13:32:28Z abeltje $
-use vars qw( $VERSION );
-$VERSION = '0.038';
+our $VERSION = '0.046';
 
+use Config;
 use Cwd;
 use File::Spec::Functions qw( :DEFAULT abs2rel rel2abs );
-use Config;
 use Test::Smoke::Util qw( get_smoked_Config skip_filter );
-require Carp;
+
 BEGIN { eval q{ use Time::HiRes qw( time ) } }
 
 my %CONFIG = (
@@ -51,11 +50,21 @@ sub HARNESS_RE1 () {
 }
 sub HARNESS_RE2() { '^\s+(\d+(?:[-\s]+\d+)*-?)$' }
 
-sub HARNESS3_RE() {
+
+sub HARNESS3_RE_EXTRA() {
+     '^\s+(\d[0-9, -]*)'
+}
+
+sub HARNESS3_RE_FAILED() {
      '^(?:
           (?:\ \ Failed\ tests?(?:\ number\(s\))?:\ \ )
-          |
-          \s+
+       )
+       (\d[0-9, -]*)'
+}
+
+sub HARNESS3_RE_TODO() {
+     '^(?:
+          (?:\ \ TODO\ passed(?:\ number\(s\))?:\ \ \ )
        )
        (\d[0-9, -]*)'
 }
@@ -81,11 +90,33 @@ Test::Smoke::Smoker - OO interface to do one smoke cycle.
 
 =head1 DESCRIPTION
 
+The Test::Smoke::Smoker module, an OO interface to do one smoke cycle.
+
+=head1 CONSTANTS
+
+=over
+
+=item BUILD_MINIPERL
+
+=item BUILD_NOTHING
+
+=item BUILD_PERL
+
+=item HARNESS3_RE_EXTRA
+
+=item HARNESS3_RE_FAILED
+
+=item HARNESS3_RE_TODO
+
+=item HARNESS_RE1
+
+=item HARNESS_RE2
+
+=back
+
 =head1 METHODS
 
-=over 4
-
-=item Test::Smoke::Smoker->new( \*GLOB, %args )
+=head2 Test::Smoke::Smoker->new( \*GLOB, %args )
 
 C<new()> takes a mandatory (opened) filehandle and some other options:
 
@@ -110,7 +141,8 @@ sub new {
     my $fh = shift;
 
     unless ( ref $fh eq 'GLOB' ) {
-        Carp::croak( sprintf "Usage: %s->new( \\*FH, %%args )", __PACKAGE__);
+        require Carp;
+        Carp::croak(sprintf "Usage: %s->new( \\*FH, %%args )", __PACKAGE__);
     }
 
     my %args_raw = @_ ? UNIVERSAL::isa( $_[0], 'HASH' ) ? %{ $_[0] } : @_ : ();
@@ -140,17 +172,29 @@ sub new {
     return $self;
 }
 
+=head2 $smoker->mark_in()
+
+Write the current timestamp with 'Start' marker to the logfile.
+
+=cut
+
 sub mark_in {
     my $self = shift;
     $self->log( sprintf "Started smoke at %d\n", time() );
 }
 
+=head2 $smoker->mark_out()
+
+Write the current timestamp with 'Stopped' marker to the logfile.
+
+=cut
+
 sub mark_out {
     my $self = shift;
     $self->log( sprintf "Stopped smoke at %d\n", time() );
 }
 
-=item Test::Smoke::Smoker->config( $key[, $value] )
+=head2 Test::Smoke::Smoker->config( $key[, $value] )
 
 C<config()> is an interface to the package lexical C<%CONFIG>,
 which holds all the default values for the C<new()> arguments.
@@ -180,7 +224,7 @@ sub config {
     return $CONFIG{ "df_$key" };
 }
 
-=item $smoker->tty( $message )
+=head2 $smoker->tty( $message )
 
 Prints a message to the default filehandle.
 
@@ -192,7 +236,7 @@ sub tty {
     print $message;
 }
 
-=item $smoker->log( $message )
+=head2 $smoker->log( $message )
 
 Prints a message to the logfile, filehandle.
 
@@ -204,7 +248,7 @@ sub log {
     print { $self->{logfh} } $message;
 }
 
-=item $smoker->ttylog( $message )
+=head2 $smoker->ttylog( $message )
 
 Prints a message to both the default and the logfile filehandles.
 
@@ -216,7 +260,7 @@ sub ttylog {
     $self->tty( @_ );
 }
 
-=item $smoker->smoke( $config[, $policy] )
+=head2 $smoker->smoke( $config[, $policy] )
 
 C<smoke()> takes a B<Test::Smoke::BuildCFG::Config> object and runs all
 the basic steps as (private) object methods.
@@ -281,7 +325,7 @@ sub smoke {
     return 1;
 }
 
-=item $smoker->make_distclean( )
+=head2 $smoker->make_distclean( )
 
 C<make_distclean()> runs C<< make -i distclean 2>/dev/null >>
 
@@ -306,7 +350,7 @@ sub make_distclean {
     }
 }
 
-=item $smoker->extra_manicheck( )
+=head2 $smoker->extra_manicheck( )
 
 C<extra_manicheck()> will only work for C<< $self->{v} > 1 >> and does
 an extra integrity check comparing F<MANIFEST> and the
@@ -331,7 +375,7 @@ sub extra_manicheck {
     }
 }
 
-=item $smoker->handle_policy( $policy, @substs );
+=head2 $smoker->handle_policy( $policy, @substs );
 
 C<handle_policy()> will try to apply the substition rules and then
 write the file F<Policy.sh>.
@@ -352,7 +396,7 @@ sub handle_policy {
     $policy->write;
 }
 
-=item $smoker->Configure( $config )
+=head2 $smoker->Configure( $config )
 
 C<Configure()> sorts out the MSWin32 mess and calls F<./Configure>
 
@@ -386,7 +430,7 @@ sub Configure {
     return -f $makefile;
 }
 
-=item $smoker->make_( )
+=head2 $smoker->make_( )
 
 C<make_()> will run make.
 
@@ -436,7 +480,7 @@ sub make_ {
         : BUILD_MINIPERL;
 }
 
-=item make_test_prep( )
+=head2 make_test_prep( )
 
 Run C<< I<make test-perp> >> and check if F<t/perl> exists.
 
@@ -454,7 +498,7 @@ sub make_test_prep {
     return $self->{is_win32} ? -f $perl : -l $perl;
 }
 
-=item $smoker->make_test( )
+=head2 $smoker->make_test( )
 
 =cut
 
@@ -502,9 +546,9 @@ sub make_test {
         unless ( $self->{run} ) {
             $self->ttylog( "bailing out (--norun)...\n" );
             next;
-	}
+        }
 
-	if  ( $self->{harnessonly} ) {
+        if  ( $self->{harnessonly} ) {
 
             $self->{harness3opts} and
                 local $ENV{HARNESS_OPTIONS} = $self->{harness3opts};
@@ -534,7 +578,7 @@ sub make_test {
     return 1;
 }
 
-=item $self->extend_with_harness( @nok )
+=head2 $self->extend_with_harness( @nok )
 
 =cut
 
@@ -591,7 +635,7 @@ sub extend_with_harness {
     }
 }
 
-=item $moker->make_test_harness
+=head2 $moker->make_test_harness
 
 Use Test::Harness (the test_harness target) to get the failing test
 information and do not bother with TEST.
@@ -621,7 +665,7 @@ sub make_test_harness {
     }
 }
 
-=item $smoker->_run_harness_target( $target, $extra )
+=head2 $smoker->_run_harness_target( $target, $extra )
 
 The command to run C<make test_harness> differs based on platform, so
 the arguments have to be passed into general routine. C<$target>
@@ -641,13 +685,20 @@ sub _run_harness_target {
 
     my $tst = $self->_make_fork( $target, $extra );
 
-    my $line;
+    my ($line, $last);
     while ( $line = <$tst> ) {
         $self->{v} > 1 and $self->tty( $line );
 
-        $line =~ /All tests successful/ and push( @failed, $line ), last;
+        # This line with timings only has to be logged to .out.
+        $line =~ / \b (?:Files | u) = .+ Tests = [0-9]+ /xi
+            and $self->log($line);
 
-        $line =~ /Failed Test\s+Stat/ and $seenheader = 1, next;
+        $last and next;
+        $line =~ /All tests successful/
+            and push( @failed, $line ), $last++, next;
+
+        $line =~ /Failed Test\s+Stat/
+            and $seenheader = 1, next;
         $seenheader or next;
 
         my( $name, $fail ) = $line =~ m/$harness_re1/;
@@ -660,17 +711,19 @@ sub _run_harness_target {
             push @failed, " " x 51 . "$fail\n";
         }
     }
+    my @dump = <$tst>; # Read trailing output from pipe
 
     close $tst or do {
         my $error = $! || ( $? >> 8);
-        Carp::carp( "\nerror while running harness target '$target': $error" );
+        require Carp;
+        Carp::carp("\nerror while running harness target '$target': $error");
     };
 
     $self->ttylog( "\n", join( "", @failed ), "\n" );
     $self->tty( "Archived results...\n" );
 }
 
-=item $smoker->_run_harness3_target( $target, $extra )
+=head2 $smoker->_run_harness3_target( $target, $extra )
 
 The command to run C<make test_harness> differs based on platform, so
 the arguments have to be passed into general routine. C<$target>
@@ -682,44 +735,90 @@ for the make program.
 sub _run_harness3_target {
     my( $self, $target, $extra ) = @_;
 
-    my $harness3_re = HARNESS3_RE();
+    my $harness3_failed = HARNESS3_RE_FAILED();
+    my $harness3_todo = HARNESS3_RE_TODO();
+    my $harness3_extra = HARNESS3_RE_EXTRA();
     my $seenheader = 0;
     my @failed = ( );
 
     my $tst = $self->_make_fork( $target, $extra );
 
     my $line;
+    my $file;
+    my $found = 0;
     while ( $line = <$tst> ) {
         $self->{v} > 1 and $self->tty( $line );
 
-        $line =~ /All tests successful/ and push( @failed, $line ), last;
+        # This line with timings only has to be logged to .out.
+        $line =~ / \b (?:Files | u) = .+ Tests = [0-9]+ /xi
+            and $self->log($line);
+
+        $line =~ /All tests successful/
+            and push( @failed, $line ), next;
 
         $line =~ /Test Summary Report/ and $seenheader = 1, next;
         $seenheader or next;
-    
-        my( $tname ) = $line =~ /^\s*(.+\.t)\s+\(Wstat/;
+
+        my( $tname ) = $line =~ /^\s*(.+(?:\.t)?)\s+\(Wstat/;
         if ( $tname ) {
-            my $dots = '.' x (60 - length $tname);
-            push @failed, "$tname${dots}FAILED\n";
+            if ($file and not $found) {
+                push @failed, "${file}??????\n";
+            }
+            my $ntest = $self->_normalize_testname( $tname );
+            my $dots = '.' x (60 - length $ntest);
+            $file = $ntest . $dots;
+            $found = 0;
             next;
         }
-    
-        my( $failed ) = $line =~ /$harness3_re/x;
+
+        my( $failed ) = $line =~ /$harness3_failed/x;
         if ( $failed ) {
+            push @failed, "${file}FAILED\n";
             push @failed, "    $failed\n";
+            $found = 1;
+            next;
+        }
+
+        my( $todo ) = $line =~ /$harness3_todo/x;
+        if ( $todo ) {
+            push @failed, "${file}PASSED\n";
+            push @failed, "    $todo\n";
+            $found = 1;
+            next;
+        }
+
+        my ( $extra ) = $line =~ /$harness3_extra/x;
+        if ( $extra) {
+            push @failed, "    $extra\n";
             next;
         }
     
         my( $parse_error ) = $line =~ /^  Parse errors: (.+)/;
         if ( $parse_error ) {
+            push @failed, "${file}FAILED\n";
             push @failed, "    $parse_error\n";
+            $found = 1;
+            next;
+        }
+
+        my( $exit_status ) = $line =~ /^  (Non-zero exit status: .+)/;
+        if ( $exit_status ) {
+            push @failed, "${file}FAILED\n";
+            push @failed, "    $exit_status\n";
+            $found = 1;
             next;
         }
     }
+    if ($file and not $found) {
+        push @failed, "${file}??????\n";
+    }
+
+    my @dump = <$tst>; # Read trailing output from pipe
 
     close $tst or do {
         my $error = $! || ( $? >> 8);
-        Carp::carp( "\nerror while running harness target '$target': $error" );
+        require Carp;
+        Carp::carp("\nerror while running harness target '$target': $error" );
     };
 
     $self->ttylog( "\n", join( "", @failed ), "\n" );
@@ -728,7 +827,10 @@ sub _run_harness3_target {
 
 sub _run_TEST_target {
     my( $self, $target, $extend ) = @_;
-    !$target and Carp::Confess( "No target in _run_TEST_target" );
+    !$target and do {
+        require Carp;
+        Carp::confess("No target in _run_TEST_target");
+    };
 
     my @nok;
     my $tst = $self->_make_fork( $target );
@@ -752,7 +854,7 @@ sub _run_TEST_target {
     }
     close $tst or do {
         my $error = $! || ( $? >> 8);
-        Carp::carp("\nError while reading test-results: $error");
+        $self->tty("\nError while reading test-results: $error");
     };
 
 #    $self->log( map { "    $_" } @nok );
@@ -768,7 +870,7 @@ sub _run_TEST_target {
     }
 }
 
-=item $self->make_minitest
+=head2 $self->make_minitest
 
 C<make> was unable to build a I<perl> executable, but managed to build
 I<miniperl>, so we do C<< S<make minitest> >>.
@@ -790,9 +892,9 @@ sub make_minitest {
     return 1;
 }
 
-=item $self->_parse_harness_output( $\%notok, $all_ok, @lines )
+=head2 $self->_parse_harness_output( $\%notok, $all_ok, @lines )
 
-Fator out the parsing of the Test::Harness output, as it seems subject
+Factor out the parsing of the Test::Harness output, as it seems subject
 to change.
 
 =cut
@@ -801,7 +903,7 @@ sub _parse_harness_output {
     my( $self, $notok, $all_ok, @lines ) = @_;
 
     grep m/^Test Summary Report/ => @lines
-        and return $self->_parse_harness3_output( $notok, $all_ok, @lines );
+        and return $self->_parse_harness3_output( $notok, $_[2], @lines );
 
     my $harness_re1 = HARNESS_RE1();
     my $harness_re2 = HARNESS_RE2();
@@ -826,7 +928,7 @@ sub _parse_harness_output {
     return $output;
 }
 
-=item $self->_parse_harness3_output( $\%notok, $all_ok, @lines )
+=head2 $self->_parse_harness3_output( $\%notok, $all_ok, @lines )
 
 Fator out the parsing of the Test::Harness 3 output, as it seems subject
 to change.
@@ -836,23 +938,52 @@ to change.
 sub _parse_harness3_output {
     my( $self, $notok, $all_ok, @lines ) = @_;
 
-    my $harness3_re = HARNESS3_RE();
+    my $harness3_failed = HARNESS3_RE_FAILED();
+    my $harness3_todo = HARNESS3_RE_TODO();
+    my $harness3_extra = HARNESS3_RE_EXTRA();
     my $seenheader = 0;
-    my $output = join "", grep defined $_ => map {
+    my $ntest = "";
+    my $file = "";
+    my $found = 0;
+
+    my @out = map {
         my $line = $_;
 
-        my( $tname ) = $line =~ /^\s*(.+\.t)\s+\(Wstat/;
-        my( $failed ) = $line =~ /$harness3_re/x;
+        my( $tname ) = $line =~ /^\s*(.+(:?\.t)?)\s+\(Wstat/;
+        my( $failed ) = $line =~ /$harness3_failed/x;
+        my( $todo ) = $line =~ /$harness3_todo/x;
+        my( $extra ) = $line =~ /$harness3_extra/x;
         my( $parse_error ) = $line =~ /^  Parse errors: (.+)/;
+        my( $exit_status ) = $line =~ /^  (Non-zero exit status: .+)/;
 
         if ( $tname ) {
-            delete $notok->{ $tname };
-            my $dots = '.' x (60 - length $tname);
-            "    $tname${dots}FAILED\n";
+            my $r;
+            if ($file and not $found) {
+                $r = "${file}??????\n";
+            }
+
+            $ntest = $self->_normalize_testname( $tname );
+            my $dots = '.' x (60 - length $ntest);
+            $file = "    $ntest${dots}";
+            $found = 0;
+            $r;
         } elsif ( $failed ) {
-            "        $failed\n";
+            delete $notok->{ $ntest };
+            $found = 1;
+            ($file . "FAILED\n", "        $failed\n");
+        } elsif ( $todo ) {
+            $found = 1;
+            ($file . "PASSED\n", "        $todo\n");
+        } elsif ($extra) {
+            "        $extra\n"
         } elsif ( $parse_error ) {
-            "        $parse_error\n";
+            delete $notok->{ $ntest };
+            $found = 1;
+            ($file . "FAILED\n", "        $parse_error\n");
+        } elsif ( $exit_status ) {
+            delete $notok->{ $ntest };
+            $found = 1;
+            ($file . "FAILED\n", "        $exit_status\n");
         } else {
             undef;
         }
@@ -862,12 +993,16 @@ sub _parse_harness3_output {
         $self->{v} and $self->tty( $_ );
         $seenheader ? $_ : '';
     } @lines;
+    if ($file and not $found) {
+        push @out, "${file}??????\n";
+    }
+    my $output = join "", grep defined $_ => @out;
 
     $_[2] = $all_ok;
     return $output;
 }
 
-=item $self->_trasnaform_testnames( @notok )
+=head2 $self->_trasnaform_testnames( @notok )
 
 C<_transform_testnames()> takes a list of testnames, as found by
 C<TEST> (testname without C<.t> suffix followed by dots and a reason)
@@ -880,31 +1015,52 @@ sub _transform_testnames {
     my( $self, @notok ) = @_;
     my %inconsistent;
     for my $nok ( @notok ) {
-        $nok =~ m!^(?:\.\.[\\/])?(\w+[\\/][-\w/\\]+)\.*(.*)! or next;
+        $nok =~ m!^((?:\.\.[\\/])?\w+[\\/][-\w/\\]+)\.*(.*)! or next;
         my( $test_name, $status ) = ( $1, $2 );
-        $test_name .= '.t';
 
-        $test_name = $test_name =~ /^(?:ext|lib|t)\b/
-            ? catfile( updir(), $test_name )
-            : catfile( updir(), 't', $test_name );
+        my $test_path = $self->_normalize_testname( $test_name );
 
-        my $test_base = catdir( $self->{ddir}, 't' );
-        $test_name = rel2abs( $test_name, $test_base );
-
-        my $test_path = abs2rel( $test_name, $test_base );
-        $test_path =~ tr!\\!/! if $self->{is_win32};
         $inconsistent{ $test_path } ||= $status;
     }
     return %inconsistent;
 }
 
-=item set_skip_tests( [$unset] )
+=head2 $smoker->_normalize_testname( $test )
+
+Normalize a testname...
+
+=cut
+
+sub _normalize_testname {
+    my( $self, $test_name ) = @_;
+
+    $test_name =~ s/\s+$//;
+    $test_name =~ /\.t$/ or $test_name .= '.t';
+    if ( $test_name !~ m|^\Q../| ) {
+        $test_name = $test_name =~ /^(?:ext|lib|t)\b/
+            ? catfile( updir(), $test_name )
+            : catfile( updir(), 't', $test_name );
+    }
+
+    my $test_base = catdir( $self->{ddir}, 'pod' );
+    $test_name = rel2abs( $test_name, $test_base );
+
+    my $test_path = abs2rel( $test_name, $test_base );
+    $test_path =~ tr!\\!/! if $self->{is_win32};
+
+    # sometimes ../t is optimized away
+    $test_path !~ m|^\.\.[\\/]| and $test_path = "../t/$test_path";
+
+    return $test_path;
+}
+
+=head2 set_skip_tests( [$unset] )
 
 Read from a MANIFEST like file, set in C<< $self->{skip_tests} >>, and
 rename the files in it with the extension F<.tskip>. If C<$unset> is
 set, they will be renamed back.
 
-=item unset_skip_tests
+=head2 unset_skip_tests
 
 Calls C<< $self->set_skip_tests( 1 ) >>.
 
@@ -925,8 +1081,10 @@ sub set_skip_tests {
         while ( $raw = <SKIPTESTS> ) {
             $raw =~ m/^\s*#/ and next;
             $raw =~ s/(\S+).*/$1/s;
-            $raw =~ m/\.t$/ or next;
-            if ( $raw =~ m{^(?:lib|ext)/} ) {
+            if ($raw !~ m/\.t$/ and $raw !~ m/test\.pl$/) {
+                next;
+            }
+            if ( $raw =~ m{^(?:lib|ext|cpan|dist)/} ) {
                 push @libext, $raw;
                 next;
             }
@@ -939,7 +1097,7 @@ sub set_skip_tests {
             my $did_mv = rename $tsrc, $tdst;
             my $error = $did_mv ? "" : " ($!)";
             $self->{v} and
-                $self->tty( sprintf "\t$raw: %sok\n", $did_mv ?  'not ' : '',
+                $self->tty( sprintf "\t%s: %sok%s\n", $raw, $did_mv ?  '' : 'not ',
                                     $error );
             -f $tdst and chmod $perms, $tdst;
         }
@@ -947,13 +1105,13 @@ sub set_skip_tests {
         @libext and $self->change_manifest( \@libext, $unset );
     } else {
         require Carp;
-        Carp::carp( "Cannot open($self->{skip_tests}): $!" );
+        Carp::carp("Cannot open($self->{skip_tests}): $!");
     }
 }
 
 sub unset_skip_tests { $_[0]->set_skip_tests( 1 ) }
 
-=item $self->change_manifest( \@tests, $unset )
+=head2 $self->change_manifest( \@tests, $unset )
 
 =cut
 
@@ -969,14 +1127,14 @@ sub change_manifest {
             unlink $mani_org;
             rename $mani_new, $mani_org;
             chmod $perms, $mani_org;
-	}
+        }
     } else {
         my $perms = (stat $mani_org)[2] & 07777;
         chmod 0755, $mani_org;
         rename $mani_org, $mani_new or do {
             chmod $perms, $mani_org;
             require Carp;
-	    Carp::carp("No skip of lib or ext tests [rename($mani_new): $!]");
+            Carp::carp("No skip of lib or ext tests [rename($mani_new): $!]");
             return;
         };
         local( *MANIO, *MANIN );
@@ -987,7 +1145,7 @@ sub change_manifest {
                     chomp $mline;
                     ( my $fn = $mline ) =~ s/^(\S+).*/$1/;
                     if ( ! grep /\Q$fn\E/ => @$tests ) {
-                        print MANIN "$fn\n";
+                        print MANIN "$mline\n";
                     } else {
                         $self->{v} and $self->tty( "\t$fn\n" );
                     }
@@ -1000,7 +1158,7 @@ sub change_manifest {
     }
 }
 
-=item $self->_run( $command[, $sub[, @args]] )
+=head2 $self->_run( $command[, $sub[, @args]] )
 
 C<_run()> returns C<< qx( $command ) >> unless C<$sub> is specified.
 If C<$sub> is defined (and a coderef) C<< $sub->( $command, @args ) >> will
@@ -1020,7 +1178,7 @@ sub _run {
     return wantarray ? @output : join " ", @output;
 }
 
-=item $self->_make( $command )
+=head2 $self->_make( $command )
 
 C<_make()> calls C<< run( "make $command" ) >>, and does some extra
 stuff to help MSWin32 (the right maker, the directory).
@@ -1031,6 +1189,7 @@ sub _make {
     my $self = shift;
     my $cmd = shift;
     $self->{makeopt} and $cmd = "$self->{makeopt} $cmd";
+    $cmd =~ m/clean/ and $cmd =~ s/-j[0-9]+\s+//;
 
     $self->{is_win32} || $self->{is_vms} or return $self->_run( "make $cmd" );
 
@@ -1056,7 +1215,7 @@ sub _make {
     return wantarray ? @output : join "", @output;
 }
 
-=item $smoker->_make_fork( $target, $extra )
+=head2 $smoker->_make_fork( $target, $extra )
 
 C<_make_fork()> opens a read pipe to the make command with C<$target>
 and C<$extra> arguments for the make command.
@@ -1084,6 +1243,7 @@ sub _make_fork {
         $ok = open TST, $cmd or $err = $!;
     }
     $ok or do {
+        require Carp;
         Carp::carp("Cannot fork '$cmd': $err");
         return 0;
     };
@@ -1091,7 +1251,7 @@ sub _make_fork {
     return *TST;
 }
 
-=item $smoker->_vms__rooted_logical
+=head2 $smoker->_vms__rooted_logical
 
 This code sets up a rooted logical C<TSP5SRC> and changes the {ddir}
 to that root.
@@ -1110,8 +1270,6 @@ sub _vms_rooted_logical {
 
 1;
 
-=back
-
 =head1 SEE ALSO
 
 L<Test::Smoke>
@@ -1129,9 +1287,9 @@ See:
 
 =over 4
 
-=item * http://www.perl.com/perl/misc/Artistic.html
+=item * L<http://www.perl.com/perl/misc/Artistic.html>
 
-=item * http://www.gnu.org/copyleft/gpl.html
+=item * L<http://www.gnu.org/copyleft/gpl.html>
 
 =back
 
