@@ -1,9 +1,9 @@
 package Test::Smoke::SourceTree;
 use strict;
 
-# $Id: SourceTree.pm 863 2005-07-21 10:28:14Z abeltje $
+# $Id$
 use vars qw( $VERSION @EXPORT_OK %EXPORT_TAGS $NOCASE );
-$VERSION = '0.007';
+$VERSION = '0.008';
 
 use File::Spec;
 use File::Find;
@@ -40,9 +40,15 @@ Test::Smoke::SourceTree - Manipulate the perl source-tree
 
     $tree->clean_from_MANIFEST;
 
-=head1 DESCRIPTION
+=head1 CONSTANTS
 
-=over 4
+=over
+
+=item ST_MISSING
+
+=item ST_UNDECLARED
+
+=back
 
 =cut
 
@@ -50,7 +56,9 @@ Test::Smoke::SourceTree - Manipulate the perl source-tree
 sub ST_MISSING()    { 1 }
 sub ST_UNDECLARED() { 0 }
 
-=item Test::Smoke::SourceTree->new( $tree_dir )
+=head1 DESCRIPTION
+
+=head2 Test::Smoke::SourceTree->new( $tree_dir )
 
 C<new()> creates a new object, this is a simple scalar containing
 C<< File::Spec->rel2abs( $tree_dir) >>.
@@ -72,7 +80,7 @@ sub new {
     return bless \$self, $class;
 }
 
-=item $tree->canonpath( )
+=head2 $tree->canonpath( )
 
 C<canonpath()> returns the canonical name for the path, 
 see L<File::Spec>.
@@ -84,7 +92,7 @@ sub canonpath {
     return File::Spec->canonpath( $$self );
 }
 
-=item $tree->rel2abs( [$base_dir] )
+=head2 $tree->rel2abs( [$base_dir] )
 
 C<rel2abs()> returns the absolute path, see L<File::Spec>.
 
@@ -95,7 +103,7 @@ sub rel2abs {
     return File::Spec->rel2abs( $$self, @_ );
 }
 
-=item $tree->abs2rel( [$base_dir] )
+=head2 $tree->abs2rel( [$base_dir] )
 
 C<abs2rel()> returns  a relative path, 
 see L<File::Spec>.
@@ -107,7 +115,7 @@ sub abs2rel {
     return File::Spec->abs2rel( $$self, @_ );
 }
 
-=item $tree->mani2abs( $file[, $base_path] )
+=head2 $tree->mani2abs( $file[, $base_path] )
 
 C<mani2abs()> returns the absolute filename of C<$file>, which should 
 be in "MANIFEST" format (i.e. using '/' as directory separator).
@@ -117,19 +125,22 @@ be in "MANIFEST" format (i.e. using '/' as directory separator).
 sub mani2abs {
     my $self = shift;
 
-    my $file = shift;
+    my $path = shift;
+    my @dirs = split m{/+}, $path;
+    my $file = pop @dirs; 
     if ( $^O eq 'VMS' ) {
         my @parts = split m/\./, $file;
         my $last = pop @parts;
         @parts and
             $file = join( "_", map { s/[^\w-]/_/g; $_ } @parts ) . ".$last";
     }
+    @dirs and $file = join '/', @dirs, $file;
     my @split_path = split m|/|, $file;
     my $base_path = File::Spec->rel2abs( $$self, @_ );
     return File::Spec->catfile( $base_path, @split_path );
 }
 
-=item $tree->mani2absdir( $dir[, $base_path] )
+=head2 $tree->mani2absdir( $dir[, $base_path] )
 
 C<mani2abs()> returns the absolute dirname of C<$dir>, which should 
 be in "MANIFEST" format (i.e. using '/' as directory separator).
@@ -144,7 +155,7 @@ sub mani2absdir {
     return File::Spec->catdir( $base_path, @split_path );
 }
 
-=item $tree->abs2mani( $file )
+=head2 $tree->abs2mani( $file )
 
 C<abs2mani()> returns the MANIFEST style filename.
 
@@ -160,7 +171,7 @@ sub abs2mani {
     return join '/', @dirs;
 }
 
-=item $tree->check_MANIFEST( @ignore )
+=head2 $tree->check_MANIFEST( @ignore )
 
 C<check_MANIFEST()> reads the B<MANIFEST> file from C<< $$self >> and
 compares it with the actual contents of C<< $$self >>.
@@ -179,7 +190,7 @@ sub check_MANIFEST {
     my %ignore = map {
         my $entry = $NOCASE ? uc $_ : $_;
         $entry => undef 
-    } ( ".patch", "MANIFEST.SKIP", @_ ), 
+    } ( ".patch", "MANIFEST.SKIP", '.git', '.gitignore', @_ ), 
       keys %{ $self->_read_mani_file( 'MANIFEST.SKIP', 1 ) };
 
     # Walk the tree, remove all found files from %manifest
@@ -188,27 +199,32 @@ sub check_MANIFEST {
     my $cwd = cwd();
     chdir $$self or die "Cannot chdir($$self): $!";
     require File::Find;
-    File::Find::find( sub {
-        -f or return;
-        my $cpath = File::Spec->canonpath( $File::Find::name );
-        my( undef, $dirs, $file ) = File::Spec->splitpath( $cpath );
-        my @dirs = grep $_ && length $_ => File::Spec->splitdir( $dirs );
-        $^O eq 'VMS' and $file =~ s/\.$//;
-        my $mani_name = join '/', @dirs, $file;
-        $NOCASE and $mani_name = uc $mani_name;
-        if ( exists $manifest{ $mani_name } ) {
-            delete $manifest{ $mani_name };
-        } else {
-            $manifest{ $mani_name } = ST_UNDECLARED
-                unless exists $ignore{ $mani_name };
-        }
-    }, '.' );
+    File::Find::find(
+        sub {
+            -f or return;
+            my $cpath = File::Spec->canonpath($File::Find::name);
+            my (undef, $dirs, $file) = File::Spec->splitpath($cpath);
+            my @dirs = grep $_ && length $_ => File::Spec->splitdir($dirs);
+            $^O eq 'VMS' and $file =~ s/\.$//;
+            my $mani_name = join '/', @dirs, $file;
+            $NOCASE and $mani_name = uc $mani_name;
+            if (exists $manifest{$mani_name}) {
+                delete $manifest{$mani_name};
+            }
+            else {
+                if (!grep $mani_name =~ /$_/, keys %ignore) {
+                    $manifest{$mani_name} = ST_UNDECLARED;
+                }
+            }
+        },
+        '.'
+    );
     chdir $cwd;
 
     return \%manifest;
 } 
 
-=item $self->_read_mani_file( $path[, $no_croak] )
+=head2 $self->_read_mani_file( $path[, $no_croak] )
 
 C<_read_mani_file()> reads the contents of C<$path> like it is a
 MANIFEST typeof file and returns a ref to hash with all values set
@@ -247,7 +263,7 @@ sub _read_mani_file {
     return \%manifest;
 }
 
-=item $tree->clean_from_MANIFEST( )
+=head2 $tree->clean_from_MANIFEST( )
 
 C<clean_from_MANIFEST()> removes all files from the source-tree that are
 not declared in the B<MANIFEST> file.
@@ -268,7 +284,7 @@ sub clean_from_MANIFEST {
     }
 }
 
-=item copy_from_MANIFEST( $dest_dir[, $verbose] )
+=head2 copy_from_MANIFEST( $dest_dir[, $verbose] )
 
 C<_copy_from_MANIFEST()> uses the B<MANIFEST> file from C<$$self>
 to copy a source-tree to C<< $dest_dir >>.
@@ -323,8 +339,6 @@ sub copy_from_MANIFEST {
 
 1;
 
-=back
-
 =head1 COPYRIGHT
 
 (c) 2002-2003, All rights reserved.
