@@ -1,9 +1,9 @@
 package Test::Smoke::Util;
 use strict;
 
-# $Id: Util.pm 1110 2007-09-23 05:21:27Z abeltje $
+# $Id$
 use vars qw( $VERSION @EXPORT @EXPORT_OK $NOCASE );
-$VERSION = '0.55';
+$VERSION = '0.58';
 
 use base 'Exporter';
 @EXPORT = qw( 
@@ -14,10 +14,10 @@ use base 'Exporter';
 );
 
 @EXPORT_OK = qw(
-    &grepccmsg &get_local_patches &set_local_patch
+    &grepccmsg &grepnonfatal &get_local_patches &set_local_patch
     &get_ncpu &get_smoked_Config &parse_report_Config 
     &get_regen_headers &run_regen_headers
-    &whereis &clean_filename
+    &whereis &clean_filename &read_logfile
     &calc_timeout &time_in_hhmm
     &do_pod2usage
     &set_vms_rooted_logical
@@ -25,6 +25,7 @@ use base 'Exporter';
 
 use Text::ParseWords;
 use File::Spec::Functions;
+use Encode qw ( decode );
 use File::Find;
 use Cwd;
 
@@ -39,9 +40,7 @@ Test::Smoke::Util - Take out some of the functions of the smoke suite.
 I've taken out some of the general stuff and put it here.
 Now I can write some tests!
 
-=over
-
-=item Configure_win32( $command[, $win32_maker[, @args]] )
+=head2 Configure_win32( $command[, $win32_maker[, @args]] )
 
 C<Configure_win32()> alters the settings of the makefile for MSWin32.
 
@@ -166,16 +165,17 @@ sub Configure_win32 {
 
     local $_;
     my %opt_map = (
-	"-Dusethreads"		=> "USE_ITHREADS",
-	"-Duseithreads"		=> "USE_ITHREADS",
-	"-Duseperlio"		=> "USE_PERLIO",
-	"-Dusemultiplicity"	=> "USE_MULTI",
-	"-Duseimpsys"		=> "USE_IMP_SYS",
-	"-Uuseimpsys"		=> "USE_IMP_SYS",
+        "-Dusethreads"          => "USE_ITHREADS",
+        "-Duseithreads"         => "USE_ITHREADS",
+        "-Duseperlio"           => "USE_PERLIO",
+        "-Dusemultiplicity"     => "USE_MULTI",
+        "-Duseimpsys"           => "USE_IMP_SYS",
+        "-Uuseimpsys"           => "USE_IMP_SYS",
         "-Dusemymalloc"         => "PERL_MALLOC",
         "-Duselargefiles"       => "USE_LARGE_FILES",
         "-Uuseshrplib"          => "BUILD_STATIC",
-	"-DDEBUGGING"		=> "USE_DEBUGGING",
+        "-UWIN64"               => "WIN64",
+        "-DDEBUGGING"           => "USE_DEBUGGING",
         "-DINST_DRV"            => "INST_DRV",
         "-DINST_TOP"            => "INST_TOP",
         "-DINST_VER"            => "INST_VER",
@@ -183,11 +183,14 @@ sub Configure_win32 {
         "-Dcf_email"            => "EMAIL",
         "-DCCTYPE"              => "CCTYPE",
         "-Dgcc_v3_2"            => "USE_GCC_V3_2",
+        "-DGCC_4XX"             => "GCC_4XX",
+        "-DGCCHELPERDLL"        => "GCCHELPERDLL",
         "-Dbccold"              => "BCCOLD",
         "-DCCHOME"              => "CCHOME",
         "-DIS_WIN95"            => "IS_WIN95",
         "-DCRYPT_SRC"           => "CRYPT_SRC",
         "-DCRYPT_LIB"           => "CRYPT_LIB",
+        "-DEXTRALIBDIRS"        => "EXTRALIBDIRS",
     );
 # %opts hash-values:
 # undef  => leave option as-is when no override (makefile default)
@@ -195,14 +198,14 @@ sub Configure_win32 {
 # (true) => enable option when no override (change value, unless
 #           $key =~ /^(?:PERL|USE)_/) (forced default)
     my %opts = (
-	USE_MULTI	=> 0,
-	USE_ITHREADS	=> 0,
-	USE_IMP_SYS	=> 0,
-	USE_PERLIO	=> 1, # useperlio should be the default!
+        USE_MULTI       => 0, # default define
+        USE_ITHREADS    => 0, # default define
+        USE_IMP_SYS     => 0, # default define
+        USE_PERLIO      => 1, # useperlio should be the default!
+        USE_LARGE_FILES => 0, # default define
         PERL_MALLOC     => 0,
-        USE_LARGE_FILES => 0,
         BUILD_STATIC    => 0,
-	USE_DEBUGGING	=> 0,
+        USE_DEBUGGING   => 0,
         INST_DRV        => undef,
         INST_TOP        => undef,
         INST_VER        => '',
@@ -210,16 +213,25 @@ sub Configure_win32 {
         EMAIL           => undef,  # used to be $smoker,
         CCTYPE          => undef,  # used to be $win32_cctype,
         USE_GCC_V3_2    => 0,
+        GCC_4XX         => 0,
+        GCCHELPERDLL    => undef,
         BCCOLD          => 0,
         CCHOME          => undef,
+        WIN64           => 1,
         IS_WIN95        => 0,
         CRYPT_SRC       => undef,
         CRYPT_LIB       => undef,
+        EXTRALIBDIRS    => undef,
     );
-#    my $def_re = qr/((?:(?:PERL|USE|IS)_\w+)|BCCOLD)/;
-    my $def_re = '((?:(?:PERL|USE|IS)_\w+)|BCCOLD)';
+
+    # $undef_re: regex for options that should be UNcommented for -Uxxx
+    my $undef_re  = qr/WIN64/;
+
+    # $def_re: regex for options that should be UNcommented for -Dxxx
+    my $def_re = qr/((?:(?:PERL|USE|IS|GCC)_\w+)|BCCOLD)/;
+
     my @w32_opts = grep ! /^$def_re/, keys %opts;
-    my $config_args = join " ", 
+    my $config_args = join " ",
         grep /^-[DU][a-z_]+/, quotewords( '\s+', 1, $command );
     push @args, "config_args=$config_args";
 
@@ -227,21 +239,21 @@ sub Configure_win32 {
     $command =~ m{^\s*\./Configure\s+(.*)} or die "unable to parse command";
     my $cmdln = $1;
     foreach ( quotewords( '\s+', 1, $cmdln ) ) {
-	m/^-[des]{1,3}$/ and next;
-	m/^-Dusedevel$/  and next;
+        m/^-[des]{1,3}$/ and next;
+        m/^-Dusedevel$/  and next;
         if ( /^-Accflags=(['"]?)(.+)\1/ ) { #emacs' syntaxhighlite
            push @buildopt, $2;
            next;
         }
         my( $option, $value ) = /^(-[DU]\w+)(?:=(.+))?$/;
-	die "invalid option '$_'" unless exists $opt_map{$option};
-	$opts{$opt_map{$option}} = $value ? $value : 1;
+        die "invalid option '$_'" unless exists $opt_map{$option};
+        $opts{$opt_map{$option}} = $value ? $value : 1;
         $option =~ /^-U/ and $opts{$opt_map{$option}} = 0;
     }
 
     # If you set one, we do all, so you can have fork()
     # unless you set -Uuseimpsys
-    unless ( $cmdln =~ /-Uuseimpsys\b/ ) {
+    if ( $cmdln !~ /-Uuseimpsys\b/ ) {
         if ( $opts{USE_MULTI} || $opts{USE_ITHREADS} || $opts{USE_IMP_SYS} ) {
             $opts{USE_MULTI} = $opts{USE_ITHREADS} = $opts{USE_IMP_SYS} = 1;
         }
@@ -254,6 +266,9 @@ sub Configure_win32 {
     # If you -Dgcc_v3_2 you 'll *want* CCTYPE = GCC
     $opts{CCTYPE} = "GCC" if $opts{USE_GCC_V3_2};
 
+    # If you -DGCC_4XX you 'll *want* CCTYPE = GCC
+    $opts{CCTYPE} = "GCC" if $opts{GCC_4XX};
+
     # If you -Dbccold you 'll *want* CCTYPE = BORLAND
     $opts{CCTYPE} = "BORLAND" if $opts{BCCOLD};
 
@@ -263,7 +278,6 @@ sub Configure_win32 {
 
     open ORG, "< $in"  or die "unable to open '$in': $!";
     open NEW, "> $out" or die "unable to open '$out': $!";
-    binmode NEW;
     my $donot_change = 0;
     while (<ORG>) {
         if ( $donot_change ) {
@@ -272,7 +286,7 @@ sub Configure_win32 {
             if (m/^\s*CFG_VARS\s*=/) {
                 my( $extra_char, $quote ) = $is_nmake
                     ? ( "\t", '"' ) : ("~", "" );
-                $_ .= join "", map "\t\t$quote$_$quote\t${extra_char}\t\\\n", 
+                $_ .= join "", map "\t\t$quote$_$quote\t${extra_char}\t\\\n",
                                    grep /\w+=/, @args;
             }
             print NEW $_;
@@ -282,7 +296,7 @@ sub Configure_win32 {
                 # We will now insert the BULDOPT lines
                 my $bo_tmpl = $win32_maker eq 'nmake'
                     ? "BUILDOPT\t= \$(BUILDOPT) %s" : "BUILDOPT\t+= %s";
-                my $buildopt = join "\n", 
+                my $buildopt = join "\n",
                                     map sprintf( $bo_tmpl, $_ ) => @buildopt;
                 $buildopt and $_ = "$buildopt\n$_\n"
             };
@@ -291,14 +305,20 @@ sub Configure_win32 {
         # Only change config stuff _above_ that line!
         if ( m/^\s*#?\s*$def_re(\s*\*?=\s*define)$/ ) {
             $_ = ($opts{$1} ? "" : "#") . $1 . $2 . "\n";
-        } elsif (m/^\s*#?\s*(CFG\s*\*?=\s*Debug)$/) {
+        }
+        elsif (m/\s*#?\s*($undef_re)(\s*\*?=\s*undef)$/) {
+            $_ = ($opts{$1} ? "#" : "") . "$1$2\n";
+        }
+        elsif (m/^\s*#?\s*(CFG\s*\*?=\s*Debug)$/) {
             $_ = ($opts{USE_DEBUGGING} ? "" : "#") . $1 . "\n";
-        } elsif (m/^\s*#?\s*(BUILD_STATIC)\s*=\s*(.*)$/) {
+        }
+        elsif (m/^\s*#?\s*(BUILD_STATIC)\s*=\s*(.*)$/) {
             my( $macro, $mval ) = ( $1, $2 );
             if ( $config_args =~ /-([UD])useshrplib\b/ ) {
                 $_ = ( $1 eq 'D' ? "#" : "" ) . "$macro = $mval\n";
             }
-        } else {
+        }
+        else {
             foreach my $cfg_var ( grep defined $opts{ $_ }, @w32_opts ) {
                 if (  m/^\s*#?\s*($cfg_var\s*\*?=)\s*(.*)$/ ) {
                     $_ =  $opts{ $cfg_var } ?
@@ -308,14 +328,14 @@ sub Configure_win32 {
                 }
             }
         }
-	print NEW $_;
+        print NEW $_;
     }
     close ORG;
     close NEW;
     return $out;
 } # Configure_win32
 
-=item set_vms_rooted_logical( $logical, $dir )
+=head2 set_vms_rooted_logical( $logical, $dir )
 
 This will set a VMS rooted logical like:
 
@@ -355,7 +375,7 @@ COMMAND
     return $result == 0;
 }
 
-=item get_cfg_filename( )
+=head2 get_cfg_filename( )
 
 C<get_cfg_filename()> tries to find a B<cfg file> and returns it.
 
@@ -376,18 +396,42 @@ sub get_cfg_filename {
     return undef;
 }
 
-=item grepccmsg( $cc, $logfile, $verbose )
+=head2 read_logfile( )
+
+Read the logfile. If an argument is passed, force to (re)read the log
+If no argument is passed, return the stored log if available otherwise
+read the logfile
+
+=cut
+
+sub read_logfile {
+    my ($logfile, $verbose) = @_;
+    return if ! defined $logfile;
+
+    open my $fh, "<", $logfile  or return undef;
+    my $log = do { local $/; <$fh> };
+    close $fh;
+
+    my $es = eval { decode("utf-8",  $log, Encode::FB_CROAK ) };
+    $@   and eval { $es = decode("cp1252",     $log, Encode::FB_CROAK ) };
+    $@   and eval { $es = decode("iso-8859-1", $log, Encode::FB_CROAK ) };
+
+    warn("Couldn't decode logfile($logfile): $@") if $@;
+    return $@ ? $log : $es;
+}
+
+=head2 grepccmsg( $cc, $logfile, $verbose )
 
 This is a port of Jarkko Hietaniemi's grepccerr script.
 
 =cut
 
 sub grepccmsg {
-    my( $cc, $logfile, $verbose ) = @_;
-    defined $logfile or return;
-    $cc ||= 'gcc';
+    my( $cc, $smokelog, $verbose ) = @_;
+    defined $smokelog or return;
+    $cc = 'gcc' if !$cc || $cc eq 'g++';
     my %OS2PAT = (
-        'aix' => 
+        'aix' =>
             # "foo.c", line n.c: pppp-qqq (W) ...error description...
             # "foo.c", line n.c: pppp-qqq (S) ...error description...
             '(^".+?", line \d+\.\d+: \d+-\d+ \([WS]\) .+?$)',
@@ -402,7 +446,7 @@ sub grepccmsg {
             # ------^
             '(^cc: (?:Warning|Error): .+?^-*\^$)',
 
-       'hpux' =>
+        'hpux' =>
             # cc: "foo.c"" line nnn: warning ppp: ...error description...
             # cc: "foo.c"" line nnn: error ppp: ...error description...
             '(^cc: ".+?", line \d+: (?:warning|error) \d+: .+?$)',
@@ -448,7 +492,7 @@ sub grepccmsg {
             # foo.c(:nn)?: undefined reference to ...
             '(^(?-s:.+?):(?: In function .+?:$|' .
                '(?: undefined reference to .+?$)|' .
-               '\d+(?:\:\d+)?: ' . '(?:warning:|error:|invalid) .+?$))',
+               '\d+(?:\:\d+)?: ' . '(?:warning:|error:|note:|invalid) .+?$))',
 
         'mswin32' => # MSVC(?:60)*
             # foo.c : error LNKnnn: error description
@@ -460,41 +504,76 @@ sub grepccmsg {
             # Warning Wnnn filename line: warning description
             # Error Ennn:: error description
             '(^(?:(?:Warning W)|(?:Error E))\d+ .+? \d+: .+?$)',
+
+	'icc' => # Intel C on Linux
+	    # pp_sys.c(4412): warning #num: text
+            #       SETi( getpriority(which, who) );
+            #       ^
+	    '(^.*?\([0-9]+\): (?:warning #[0-9]+|error): .+$)',
+	'icpc' => # Intel C++
+	    '(^.*?\([0-9]+\): (?:warning #[0-9]+|error): .+$)',
     );
     exists $OS2PAT{ lc $cc } or $cc = 'gcc';
     my $pat = $OS2PAT{ lc $cc };
 
     my( $indx, %error ) = ( 1 );
-    my $smokelog = '';
-    local *LOGFH;
-    if ( open LOGFH, "< $logfile" ) {
-        $verbose and print "Reading logfile '$logfile'\n";
-        local $/;
-        $smokelog = <LOGFH>;
-        close LOGFH;
+    if ($smokelog) {
         $verbose and print "Pattern($cc): /$pat/\n";
     } else {
-        $verbose and print "Skipping '$logfile' '$!'\n";
-        $error{ "Couldn't examine '$logfile' for compiler warnings." } = 1;
+        $error{ "Couldn't examine logfile for compiler warnings." } = 1;
     }
 
-    $error{ $1 } ||= $indx++ while $smokelog =~ /$pat/mg;
+    while ($smokelog =~ m/$pat/mg) {
+        (my $msg = $1) =~ s/^\s+//;
+        $msg =~ s/[\s\r\n]+\Z//;
 
-    # I need to think about this IRIX/$Config{cc} thing
-#    if ($cc eq 'irix') {
-#        if ($Config{cc} =~ /-n32|-64/) {
-#    	     delete @error{ grep { /cc-(?:1009|1110|1047) / } keys %error };
-#        }
-#    }
+        # Skip known junk from Configure
+        $msg =~ m{^try\.c:[ :0-9]+\bwarning:} and next;
 
-    my @errors = map {
-        chomp; $_;
-    } sort { $error{ $a } <=> $error{ $b } } keys %error;
+        # We need to think about this IRIX/$Config{cc} thing
+        # $cc eq "irix" && $Config{cc} =~ m/-n32|-64/ &&
+        #     $msg =~ m/cc-(?:1009|1110|1047) / and next;
+
+        $error{ $msg } ||= $indx++;
+    }
+
+    my @errors = sort { $error{ $a } <=> $error{ $b } } keys %error;
 
     return wantarray ? @errors : \@errors;
 }
 
-=item get_local_patches( $ddir )
+=head2 grepnonfatal( $cc, $logfile, $verbose )
+
+This is a way to find known failures that do not cause the tests to
+fail but are important enough to report, like being unable to install
+manual pages.
+
+=cut
+
+sub grepnonfatal {
+    my( $cc, $smokelog, $verbose ) = @_;
+    $smokelog or return;
+
+    my( $indx, %error ) = ( 1 );
+
+    my $kf = qr{
+        # Pod::Man is not available: Can't load module Encode, dynamic loading not available in this perl.
+        (\b (\S+) (?-x: is not available: Can't load module )
+            (\S+?) , (?-x: dynamic loading not available) )
+    }xi;
+
+    while ($smokelog =~ m{$kf}g) {
+        my $fail = $1; # $2 = "Pod::Man", $3 = "Encode"
+
+        $error{ $fail } ||= $indx++;
+    }
+
+    my @errors = sort { $error{ $a } <=> $error{ $b } } keys %error;
+
+    return wantarray ? @errors : \@errors;
+}
+
+=head2 get_local_patches( $ddir )
 
 C<get_local_patches()> reads F<patchlevel.h> to scan for the locally
 applied patches array.
@@ -531,7 +610,7 @@ sub get_local_patches {
     return @lpatches;
 }
 
-=item set_local_patch( $ddir, @descr )
+=head2 set_local_patch( $ddir, @descr )
 
 Copy the code from F<patchlevel.h>. Older (pre 5.8.1) perls do not
 have it and it doesn't work on MSWin32.
@@ -582,7 +661,7 @@ sub set_local_patch {
     return 1;
 }
 
-=item get_config( $filename )
+=head2 get_config( $filename )
 
 Read and parse the configuration from file, or return the default
 config.
@@ -673,7 +752,7 @@ sub get_config {
     return @cnf_stack;
 }
 
-=item get_patch( [$ddir] )
+=head2 get_patch( [$ddir] )
 
 Try to find the patchlevel, look for B<.patch> or try to get it from
 B<patchlevel.h> as a fallback.
@@ -688,11 +767,15 @@ sub get_patch {
     local *DOTPATCH;
     my $patch_level = '?????';
     if ( open DOTPATCH, "< $dot_patch" ) {
-        chomp( $patch_level = <DOTPATCH> );
+        chomp( $patch_level = <DOTPATCH> || '' );
         close DOTPATCH;
-        return $patch_level if $patch_level =~ /-RC\d+$/;
-        $patch_level =~ tr/0-9//cd;
-        return $1 if $patch_level =~/^([0-9]+)$/;
+
+        if ( $patch_level ) {
+            my ($branch, $date, $sha, $describe) = split(' ', $patch_level);
+            ( my $short_describe = $describe ) =~ s/^GitLive-//;
+            return [$sha, $short_describe, $branch]
+        }
+        return [ '' ];
     }
 
     # There does not seem to be a '.patch', try 'patchlevel.h'
@@ -714,10 +797,10 @@ sub get_patch {
             }
         }
     }
-    return $patch_level;
+    return [ $patch_level ];
 }
 
-=item version_from_patchlevel_h( $ddir )
+=head2 version_from_patchlevel_h( $ddir )
 
 C<version_from_patchlevel_h()> returns a "dotted" version as derived 
 from the F<patchlevel.h> file in the distribution.
@@ -743,17 +826,17 @@ sub version_from_patchlevel_h {
             return "$revision.$version$subversion";
         }
 
-        $revision   = $patchlevel =~ /^#define PERL_REVISION\s+(\d+)/m 
+        $revision   = $patchlevel =~ /^#define PERL_REVISION\s+(\d+)/m
                     ? $1 : '?';
         $version    = $patchlevel =~ /^#define PERL_VERSION\s+(\d+)/m
                     ? $1 : '?';
-        $subversion = $patchlevel =~ /^#define PERL_SUBVERSION\s+(\d+)/m 
+        $subversion = $patchlevel =~ /^#define PERL_SUBVERSION\s+(\d+)/m
                     ? $1 : '?';
     }
     return "$revision.$version.$subversion";
 }
  
-=item get_ncpu( $osname )
+=head2 get_ncpu( $osname )
 
 C<get_ncpu()> returns the number of available (online/active/enabled) CPUs.
 
@@ -767,6 +850,8 @@ is returned.
 In the first case (where we really have no idea how to proceed),
 also a warning (C<get_ncpu: unknown operating system>) is sent to STDERR.
 
+=over
+
 =item B<WARNINGS>
 
 If you get the warning C<get_ncpu: unknown operating system>, you will
@@ -777,6 +862,8 @@ superuser (administrator) rights.
 
 Thanks to Jarkko Hietaniemi for donating this!
 
+=back
+
 =cut
 
 sub get_ncpu {
@@ -803,7 +890,7 @@ sub get_ncpu {
             my @output = grep /^processor/ => `ioscan -fnkC processor`;
             $cpus = scalar @output;
             last OS_CHECK;
-	};
+        };
 
         /irix/i && do {
             my @output = grep /\s+processors?$/i => `hinv -c processor`;
@@ -819,7 +906,7 @@ sub get_ncpu {
             }
             $cpus = @output ? scalar @output : '';
             last OS_CHECK;
-	};
+        };
 
         /solaris|sunos|osf/i && do {
             my @output = grep /on-line/ => `psrinfo`;
@@ -839,6 +926,15 @@ sub get_ncpu {
             last OS_CHECK;
         };
 
+        /haiku/i && do {
+            eval { require Haiku::SysInfo };
+            if (!$@) {
+                my $hsi = Haiku::SysInfo->new();
+                $cpus = $hsi->cpu_count();
+                last OS_CHECK;
+            }
+        };
+
         $cpus = "";
         require Carp;
         Carp::carp( "get_ncpu: unknown operationg system" );
@@ -847,7 +943,7 @@ sub get_ncpu {
     return $cpus ? sprintf( "%s cpu%s", $cpus, $cpus ne "1" ? 's' : '' ) : "";
 }
 
-=item get_smoked_Config( $dir, @keys )
+=head2 get_smoked_Config( $dir, @keys )
 
 C<get_smoked_Config()> returns a hash (a listified hash) with the
 specified keys. It will try to find F<lib/Config.pm> to get those
@@ -928,11 +1024,11 @@ sub get_smoked_Config {
     return %Config;
 }
 
-=item parse_report_Config( $report )
+=head2 parse_report_Config( $report )
 
 C<parse_report_Config()> returns a list attributes from a smoke report.
 
-    my( $version, $plevel, $os, $osvers, $archname, $summary ) = 
+    my( $version, $plevel, $os, $osvers, $archname, $summary, $branch ) = 
         parse_report_Config( $rpt );
 
 =cut
@@ -940,19 +1036,23 @@ C<parse_report_Config()> returns a list attributes from a smoke report.
 sub parse_report_Config {
     my( $report ) = @_;
 
-    my $version  = $report =~ /^Automated.*for (.+) patch/ ? $1 : '';
-    my $plevel   = $report =~ /^Automated.*patch (\d+(?:\.\d+\.\d+-RC\d+)?)/
+    my $branch   = $report =~ /^Automated.+branch (.+?) / ? $1 : 'blead';
+    my $version  = $report =~ /^Automated.*for(?: branch \S+)? (.+) patch/ ? $1 : '';
+    my $plevel   = $report =~ /^Automated.+?(\S+)$/m
         ? $1 : '';
+    if ( !$plevel ) {
+        $plevel = $report =~ /^Auto.*patch\s+\S+\s+(\S+)/ ? $1 : '';
+    }
     my $osname   = $report =~ /\bon\s+(.*) - / ? $1 : '';
     my $osvers   = $report =~ /\bon\s+.* - (.*)/? $1 : '';
     $osvers =~ s/\s+\(.*//;
     my $archname = $report =~ /:.* \((.*)\)/ ? $1 : '';
     my $summary  = $report =~ /^Summary: (.*)/m ? $1 : '';
 
-    return ( $version, $plevel, $osname, $osvers, $archname, $summary );
+    return ( $version, $plevel, $osname, $osvers, $archname, $summary, $branch );
 }
 
-=item get_regen_headers( $ddir )
+=head2 get_regen_headers( $ddir )
 
 C<get_regen_headers()> looks in C<$ddir> to find either 
 F<regen_headers.pl> or F<regen.pl> (change 18851).
@@ -976,7 +1076,7 @@ sub get_regen_headers {
     return; # Should this be "make regen_headers"?
 }
 
-=item run_regen_headers( $ddir, $verbose );
+=head2 run_regen_headers( $ddir, $verbose );
 
 C<run_regen_headers()> gets its executable from C<get_regen_headers()>
 and opens a pipe from it. warn()s on error.
@@ -1008,7 +1108,7 @@ sub run_regen_headers {
     return 1;
 }
 
-=item whereis( $prog )
+=head2 whereis( $prog )
 
 Try to find an executable instance of C<$prog> in $ENV{PATH}.
 
@@ -1029,13 +1129,15 @@ sub whereis {
     foreach my $dir ( @path ) {
         foreach my $ext ( @pext ) {
             my $fname = File::Spec->catfile( $dir, "$prog$ext" );
-            return $fname if -x $fname;
+            if ( -x $fname ) {
+                return $fname =~ /\s/ ? qq/"$fname"/ : $fname;
+            }
         }
     }
     return '';
 }
 
-=item vms_whereis( $prog )
+=head2 vms_whereis( $prog )
 
 First look in the SYMBOLS to see if C<$prog> is there.
 Next look in the KFE-table C<INSTALL LIST> if it is there.
@@ -1080,7 +1182,7 @@ sub vms_whereis {
     return '';
 }
 
-=item clean_filename( $fname )
+=head2 clean_filename( $fname )
 
 C<clean_filename()> basically returns a vmsify() type of filename for
 VMS, and returns an upcase filename for case-ignorant filesystems.
@@ -1100,7 +1202,7 @@ sub clean_filename {
     return $NOCASE ? uc $fname : $fname;
 }
 
-=item calc_timeout( $killtime[, $from] )
+=head2 calc_timeout( $killtime[, $from] )
 
 C<calc_timeout()> calculates the timeout in seconds. 
 C<$killtime> can be one of two formats:
@@ -1141,7 +1243,7 @@ sub calc_timeout {
     return $timeout;
 }
 
-=item time_in_hhmm( $diff )
+=head2 time_in_hhmm( $diff )
 
 Create a string telling elapsed time in days, hours, minutes, seconds
 from the number of seconds.
@@ -1170,7 +1272,7 @@ sub time_in_hhmm {
     return join " ", @parts;
 }
 
-=item do_pod2man( %pod2usage_options )
+=head2 do_pod2usage( %pod2usage_options )
 
 If L<Pod::Usage> is there then call its C<pod2usage()>.
 In the other case, print the general message passed with the C<myusage> key.
@@ -1198,7 +1300,7 @@ EO_MSG
     }
 }
 
-=item skip_config( $config ) 
+=head2 skip_config( $config ) 
 
 Returns true if this config should be skipped.
 C<$config> should be a B<Test::Smoke::BuildCFG::Config> object.
@@ -1219,7 +1321,7 @@ sub skip_config {
     return $skip;
 }
 
-=item skip_filter( $line )
+=head2 skip_filter( $line )
 
 C<skip_filter()> returns true if the filter rules apply to C<$line>.
 
@@ -1229,6 +1331,8 @@ sub skip_filter {
     local( $_ ) = @_;
     # Still to be extended
     return m,^ *$, ||
+    m,^\t, ||
+    m,^PERL=./perl\s+./runtests choose, ||
     m,^	AutoSplitting, ||
     m,^\./miniperl , ||
     m,^\s*autosplit_lib, ||
@@ -1245,7 +1349,7 @@ sub skip_filter {
     m,^lib/ftmp-security....File::Temp::_gettemp: Parent directory \((\.|/tmp/)\) is not safe, ||
     m,^File::Temp::_gettemp: Parent directory \((\.|/tmp/)\) is not safe, ||
     m,^ok$, ||
-    m,^[-a-zA-Z0-9_/]+\.*(ok|skipping test on this platform)$, ||
+    m,^[-a-zA-Z0-9_/.]+\s*\.*\s*(ok|skipped|skipping test on this platform)$, ||
     m,^(xlc|cc_r) -c , ||
 #    m,^\s+$testdir/, ||
     m,^sh mv-if-diff\b, ||
@@ -1276,20 +1380,18 @@ sub skip_filter {
     m,dmake\.exe:?\s+-S, ||
     m,^\s+\d+/\d+ skipped: , ||
     m,^\s+all skipped: , ||
-    m,\.+skipped$, ||
     m,^\s*pl2bat\.bat [\w\\]+, ||
     m,^Making , ||
     m,^Skip , ||
-    m,^Creating library file: libExtTest\.dll\.a,
+    m,^Creating library file: libExtTest\.dll\.a, ||
+    m,^cc: warning 983: ,
 }
 
 1;
 
-=back
-
 =head1 COPYRIGHT
 
-(c) 2001-2003, All rights reserved.
+(c) 2001-2014, All rights reserved.
 
   * H. Merijn Brand <h.m.brand@hccnet.nl>
   * Nicholas Clark <nick@unfortu.net>
