@@ -1,9 +1,9 @@
 package TestLib;
 use strict;
 
-# $Id: TestLib.pm 992 2006-06-04 08:52:09Z abeltje $
+# $Id$
 use vars qw( $VERSION @EXPORT );
-$VERSION = '0.05';
+$VERSION = '0.06';
 
 use Test::Smoke::Util qw( whereis );
 
@@ -35,7 +35,6 @@ What is in here?
 =cut
 
 use Config;
-use Carp;
 use File::Find;
 use File::Spec::Functions qw( :DEFAULT abs2rel rel2abs
                               splitdir splitpath catpath);
@@ -161,7 +160,7 @@ sub find_a_patch {
     my $patch_bin;
     foreach my $patch (qw( gpatch npatch patch )) {
         $patch_bin = whereis( $patch ) or next;
-        my $version = `$patch_bin --version`;
+        my $version = `$patch_bin --version 2>&1`;
         $? or return $patch_bin;
     }
 }
@@ -200,7 +199,7 @@ sub do_unzip {
         require Compress::Zlib;
         my $unzipper = Compress::Zlib::gzopen( $uzfile, 'rb' ) or do {
             require Carp;
-            Carp::carp "Can't open '$uzfile': $Compress::Zlib::gzerrno";
+            Carp::carp( "Can't open '$uzfile': $Compress::Zlib::gzerrno" );
             return undef;
         };
 
@@ -209,7 +208,7 @@ sub do_unzip {
 
         unless ( $Compress::Zlib::gzerrno == Compress::Zlib::Z_STREAM_END() ) {
             require Carp;
-            Carp::carp "Error reading '$uzfile': $Compress::Zlib::gzerrno" ;
+            Carp::carp( "Error reading '$uzfile': $Compress::Zlib::gzerrno" );
         }
 
         $unzipper->gzclose;
